@@ -1,15 +1,11 @@
-"drCCAcombine" <-
-function(datasets,reg=0,nfold=3,nrand=50)
+drCCAcombine <- function(datasets, reg = 0, nfold = 3, nrand = 50)
 {
 
-  mat <- datasets #list of data matrices
-
-  reg <- reg #regularization value
-
+  mat <- datasets # list of data matrices
+  #reg: regularization value
 
   n <- nfold # number of cross validations
 
-
   iter <- nrand # number of iterations for random matrix generation
 
   if(reg < 0){reg <- 0}
