@@ -6,9 +6,13 @@
 # "Reality is merely an illusion, albeit a very persistent one."
 # - Albert Einstein
         
-
+	
 get.W <- function (vec, Dim) {
 
+  # (C) 2008-2011 Leo Lahti and Olli-Pekka Huovilainen          
+  # All rights reserved. 
+  # FreeBSD License (keep this notice)     
+
   # Convert parameter vector into matrices Wx and Wy
   W <- list()
   W$X <- array(vec[1:(Dim$X*Dim$Z)], dim = c(Dim$X,Dim$Z))
@@ -18,9 +22,16 @@ get.W <- function (vec, Dim) {
   list(W = W, T = T)
 }
 
-get.W2 <- function (vec, Dim) {
+
+
+get.W.nonneg <- function (vec, Dim) {
+
+  # (C) 2008-2011 Leo Lahti and Olli-Pekka Huovilainen          
+  # All rights reserved. 
+  # FreeBSD License (keep this notice)     
 
   # Convert parameter vector into matrices Wx and Wy
+  # in general Wx != Wy
 
   # Note that we always assume that W is positive Therefore remove the
   #sign here to speed up optimization
@@ -28,7 +39,7 @@ get.W2 <- function (vec, Dim) {
   
   # NOTE: assumes dx = dy  
   W <- list()
-  nx <- (Dim$X*Dim$Z)
+  nx <- Dim$X*Dim$Z
   W$X <- array(vec[1:nx], dim = c(Dim$X, Dim$Z))
   W$Y <- array(vec[(nx + 1):length(vec)], dim = c(Dim$Y, Dim$Z))
   W$total <- rbind(W$X, W$Y)
@@ -37,19 +48,46 @@ get.W2 <- function (vec, Dim) {
 
 }
 
-get.W4 <- function (vec, Dim) {
+get.W.nonneg.identical <- function (vec, Dim) {  
 
-  # Convert parameter vector into matrices Wx and Wy
-  # assumes Wx = Wy
+  # (C) 2008-2011 Leo Lahti and Olli-Pekka Huovilainen          
+  # All rights reserved. 
+  # FreeBSD License (keep this notice)     
 
-  # NOTE: assumes dx = dy  
-  W <- list()
-  W$X <- W$Y <- array(abs(vec), dim = c(Dim$X, Dim$Z))
-  # Note that we always assume that W is positive
-  # Therefore remove the sign here to speed up optimization
-  # FIXME: later, allow individual scales for Wx, Wy?
+  # Convert parameter vector into matrix               
+  # Note that we always assume that W is positive          
+  # Therefore remove the sign here to speed up optimization    
+  
+  array(abs(vec), dim = c(Dim$X, Dim$Z))              
   
-  W
-
 }
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
